// Knocker.cpp: implementation of the Knocker class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Knocker.h"

#include <string.h>



void printError(TCHAR* string);
void updateStatus(TCHAR* string);

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////


/*********************************************************************************\
*
\*********************************************************************************/
DWORD WINAPI knockThreadFunction(LPVOID knockerInstance)
{
	Knocker*	knocker;
	int			i;
	int			j;
	bool		error;


	error=false;

	knocker=(Knocker*)knockerInstance;


	_stprintf(knocker->status, _T("Playing sequence to %d.%d.%d.%d:%d: \n"),
		                 knocker->destIp[0], knocker->destIp[1], knocker->destIp[2], knocker->destIp[3], knocker->destPort);
	updateStatus(knocker->status);
	


	i=0;
	while ((i<MAX_SEQUENCE) && !error)
	{
		j=0;
		while (j<knocker->knockSequence[i])
		{
			error=knocker->createSynPacket();
			knocker->sendSynPacket();
			_tcscat(knocker->status, _T("K"));
			j++;
		}
		updateStatus(knocker->status);
		if (i<MAX_SEQUENCE-1)
		{
			Sleep(knocker->timeWindow);
			_tcscat(knocker->status, _T("."));
			updateStatus(knocker->status);
		}

		i++;
	}
	_tcscat(knocker->status, _T("\nOpened\n"));
	updateStatus(knocker->status);
	Sleep(knocker->timeWindow);
	_tcscat(knocker->status, _T("Closed\n"));
	updateStatus(knocker->status);


	knocker->knockThreadId=NULL;

	return 0;
}


/*********************************************************************************\
*
\*********************************************************************************/
DWORD WINAPI packetCatchThreadFunction(LPVOID knockerInstance)
{
	Knocker*			knocker;
	bool				error;
	bool				exit;
	struct pcap_pkthdr	*header;
	const u_char		*pkt_data;
	char				destAddrString[20];
	char				filterString[100];
	int					retvalue;

	error=false;

	knocker=(Knocker*)knockerInstance;


	// ***** Now apply the filter *****
	// Receive only incoming packets from the target host
	// Filter details..
	// Protocol : TCP
	// Source Address  : Target Machine
	// Destination port : same as source machine port

	sprintf(destAddrString,"%d.%d.%d.%d",	knocker->targetMachine[0],
											knocker->targetMachine[1],
											knocker->targetMachine[2],
											knocker->targetMachine[3]);
//	sprintf(filterString,"src net %s and tcp src port %d",destAddrString, knocker->destPort);
	sprintf(filterString,"src net %s",destAddrString);
	

	pcap_freecode(&knocker->fcode);
	knocker->applyFilter(filterString);


	exit=false;
	while(!exit)
	{

		retvalue=pcap_next_ex(knocker->hdev,&header,&pkt_data);
	
		if(retvalue<0)				// error while reading
		{
			printError(_T("Error capturing packets"));
			error=true;
			exit=true;
		}
		else if (retvalue==1)		// correctly catched packet
		{

			error=false;
		}
		else if (retvalue==0)		// timed out
		{
		}
	}

	return 0;
}

/*********************************************************************************\
*
\*********************************************************************************/
Knocker::Knocker()
{
	WORD wVersionRequested;
	WSADATA wsaData;
	int err;

	
	
	isOpen=false;
	openDeviceIndex=-1;
	knockThreadId=NULL;
	listenThreadId=NULL;

	// To do: initialise devices[] array




 
	wVersionRequested = MAKEWORD( 2, 2 );
 
	err = WSAStartup( wVersionRequested, &wsaData );
	if ( err != 0 ) 
	{
        printError(_T("Unable to start WSA stuff"));
		return;
	}
 
}

/*********************************************************************************\
*
\*********************************************************************************/
Knocker::~Knocker()
{

    WSACleanup( );
}


/*********************************************************************************\
*
\*********************************************************************************/
bool Knocker::setDestinationIp	(TCHAR* ip)
{

	bool		notAnIp;
	bool		error;
	char*		string;
	HOSTENT		*h;

	error=false;
	notAnIp=true;

	string=tcharToChar(ip, strlen(ip)+1);

	// first rudimentary check: IP addresses do not exceed 15 characters
	if (strlen(string)<15)
	{
		notAnIp=this->stringToByteArray(string, destIp, 4, '.');
	}

	// Check if the string was an IP address
	// If not, it was a host name. Convert it to an IP
	if (notAnIp)
	{
		h=::gethostbyname(string);
		memcpy(destIp,h->h_addr_list[0],4);
	}

	return error;
}



/*********************************************************************************\
*
\*********************************************************************************/
bool Knocker::setDestinationPort	(TCHAR* port)
{
	bool error;

	error=false;

	destPort=_ttoi(port);

	if (destPort<PORT_MIN || destPort>PORT_MAX)
	{
		printError(_T("Invalid port"));
		error=true;
	}

	return error;
}

/*********************************************************************************\
*
\*********************************************************************************/
bool Knocker::setKnockSequence	(TCHAR* sequence)
{
	int  i;
	bool error;

	if (_tcslen(sequence)<MAX_SEQUENCE)
	{
		error=true;
	}
	else
	{
		error=false;
		i=0;
		while ((i<MAX_SEQUENCE) && (!error))
		{
			knockSequence[i]=sequence[i]-'0';
			if ((knockSequence[i]<SEQUENCE_MINVALUE) || (knockSequence[i]>SEQUENCE_MAXVALUE))
			{
				error=true;
			}
			i++;
		}
	}
	return error;
}


/*********************************************************************************\
*
\*********************************************************************************/
bool Knocker::setTimeWindow(TCHAR* window)
{
	bool error;

	error=false;

	timeWindow=_ttoi(window);

	if (timeWindow<TIMEWINDOW_MIN || timeWindow>TIMEWINDOW_MAX)
	{
		printError(_T("Invalid time window"));
		error=true;
	}

	return error;
}

/*********************************************************************************\
*
\*********************************************************************************/
bool Knocker::setMacAddress(int index, TCHAR* macAddress)
{
	bool error;
	int  i;

	error=false;

	int dummy[6];

	_stscanf(macAddress, _T("%02X:%02X:%02X:%02X:%02X:%02X"),	&dummy[0],
																&dummy[1],
																&dummy[2],
																&dummy[3],
																&dummy[4],
																&dummy[5]);
	i=0;
	while (i<6)
	{
		devices[index].ihaMac[i]=(BYTE)dummy[i];
		i++;
	}

	return error;
}
	

/*********************************************************************************\
*
\*********************************************************************************/
TCHAR* Knocker::getMacAddress(int index)
{
	_stprintf(macString, _T("%02X:%02X:%02X:%02X:%02X:%02X"),	devices[index].ihaMac[0],
																devices[index].ihaMac[1],
																devices[index].ihaMac[2],
																devices[index].ihaMac[3],
																devices[index].ihaMac[4],
																devices[index].ihaMac[5]);

	return macString;
}


/*********************************************************************************\
*
\*********************************************************************************/
bool Knocker::openDevice(int index)
{
	char	err[PCAP_ERRBUF_SIZE];
	bool	error;
    	
	error=false;

	// if the device is already open
	// close the device...
	if(isOpen==TRUE)
	{
		closeDevice();	
	}	
	
	hdev=pcap_open_live( LPCTSTR(devices[index].wpcName), //name of the device
		                 65535,		 //Capture whole packet
	                     1,			 //promiscous mode  
						 1000,       //read timeout in ms
						 err 
						 );

	

	if(hdev!=NULL)
	{
	/*
		//check the link layer to support only ethernet
		if(pcap_datalink(hdev)!=DLT_EN10MB)
		{
			printError(_T("\n Error : Only Ethernet Network is supported"));
			return FALSE;
		}

	*/

		openDeviceIndex=index;
		isOpen=TRUE;
	}
	else
	{
		error=true;
		printError(_T("Cannot open device!!"));
	}



	return error;

}


/*********************************************************************************\
*
\*********************************************************************************/
bool  Knocker::closeDevice()
{
	bool error;

	error=false;

	if(isOpen==TRUE)
	{

		pcap_freealldevs(devlist);
	
		pcap_close(hdev);

		hdev=NULL;
	}
	
	openDeviceIndex=-1;
	isOpen=FALSE;


	return error;
}



/*********************************************************************************\
*
\*********************************************************************************/
bool Knocker::preparePacket		()
{

	bool			error;


	int index=0;
	int repcount=0;
	

	error=false;




	
	//If capture device not opened then return ...
	if(isOpen==FALSE)
	{
		error=true;
	}

	


	
	
	// Don't use real host address as source address
	// Just change the last byte of source ip address
	// This is called IP Spoofing...
//	memcpy(hostIp,srcIp,4);
//	hostIp[3]=::rand()%254+1;

	
	if (!error)
	{
		// Check for destination address = 127.0.0.1
		// If so then assign proper address of the host to destination
		if(destIp[0]==127 && destIp[1]==0 && destIp[2]==0 && destIp[3]==1)
		{
			memcpy(destIp,devices[openDeviceIndex].ihaIp,4);
		}
		// Determine the MAC address of the destination machine
		error=getDestMacAddress();

		if (error)
		{
			printError(_T("Unable to determine the destination MAC address"));
			error=true;
		}
	}
	return error;

}

/*********************************************************************************\
*
\*********************************************************************************/
bool Knocker::createSynPacket()
{
	eth_header		eh;
	ip_header		ih;
	tcp_header		th;
	pseudo_header	psh;
	bool			error;
	char			dummyData[4];


	error=false;

/*
	// ***** Now apply the filter *****
	// Receive only incoming packets from the target host
	// Filter details..
	// Protocol : TCP
	// Source Address  : Target Machine
	// Destination port : same as source machine port

	sprintf(destaddr,"%d.%d.%d.%d",destIp[0],destIp[1],destIp[2],destIp[3]);
	sprintf(str,"src net %s and tcp port 6000",destaddr);
	
	ApplyFilter(str);
*/	




	// Prepare Ethernet Header , IP Header and TCP Header in Advance

	
	/*******************  Ethernet Header ******************/

	memset(&eh,0,sizeof(eh));
	
	//Source MAC address
	memcpy(eh.shost,devices[openDeviceIndex].ihaMac,6);
	
	//Destination MAC address
	memcpy(eh.dhost,destMac,6);
	
	eh.type=htons(0x0800); //IP Frames

	memcpy(&synPacket,&eh,ETHER_LENGTH);


	
	/*******************  IP Header ******************/
	
	memset(&ih,0,sizeof(ih));
	
	//source address
	memcpy(&ih.saddr,devices[openDeviceIndex].ihaIp,4);
		
	//destination address
	memcpy(&ih.daddr,destIp,4);
	
	ih.proto=PROTO_TCP;         

	ih.flags_fo=htons(0x4000);			//For TCP Flag fixed
//	ih.flags_fo=htons(0x0000);			//For ICMP Flag fixed
	ih.identification=htons(0x0150);    //Any number
	
	
	ih.tlen=htons(IP_LENGTH+TCP_LENGTH);      //Depends upon data and tcp protocol
	ih.tos=0;
	ih.ttl=32;
	ih.ver_ihl=0x45;  //Version (v4) and  header length(5 nibbles)
	ih.crc=0;
	ih.crc=computeChecksum((u_short *)&ih,IP_LENGTH);

	memcpy(&synPacket[ETHER_LENGTH],&ih,IP_LENGTH);

	
	
	
	/***************  TCP Header ****************************/

	  
	memset(&th,0,sizeof(th));
	th.sport=htons(1000+(rand()%1000));		// source port, random between 8000-9000
	th.dport=htons(destPort);				// dest port changes dynamically
	th.ackno=htonl(0);
//	th.seqno=htonl(5555l);
	th.seqno=htonl(1l);
	th.flag=TCP_SYN;        // SYN packet
	th.offset=0x50;         // data offset
	th.uptr=0;
	th.win=htons(512) ;
	th.checksum=0;
	
	//set up the pseudo header 
	memset(&psh,0,sizeof(psh));
	psh.saddr=ih.saddr;
	psh.daddr=ih.daddr;
	psh.proto=ih.proto;
	psh.zero=0;
	psh.tcp_len=htons(TCP_LENGTH);				//Length of Tcp header + Data in OCTATES
	memcpy(&psh.tcp,&th,TCP_LENGTH);
	
	th.checksum=computeChecksum((u_short*)&psh,TCP_LENGTH+PSEUDO_LENGTH);


	/***************  data******* ****************************/
	dummyData[0]=2;

	//copy new tcp header into pseudo header
	memcpy(&psh.tcp,&th,TCP_LENGTH);
		
	//Compute Checksum for TCP Header
	th.checksum=computeChecksum((u_short*)&psh,TCP_LENGTH+PSEUDO_LENGTH);

	//Copy TCP Header into the frame at the end of IP Packet
	memcpy(&synPacket[ETHER_LENGTH+IP_LENGTH],&th,TCP_LENGTH);

	//Copy TCP Header into the frame at the end of IP Packet
	memcpy(&synPacket[ETHER_LENGTH+IP_LENGTH+TCP_LENGTH], dummyData,4);

	
	return error;

}


/*********************************************************************************\
*
\*********************************************************************************/
bool Knocker::createResetPacket(int srcPort)
{
	eth_header		eh;
	ip_header		ih;
	tcp_header		th;
	pseudo_header	psh;
	bool			error;


	error=false;

	// Prepare Ethernet Header , IP Header and TCP Header 

	
	/*******************  Ethernet Header ******************/

	memset(&eh,0,sizeof(eh));
	
	//Source MAC address
	memcpy(eh.shost,devices[openDeviceIndex].ihaMac,6);
	
	//Destination MAC address
	memcpy(eh.dhost,destMac,6);
	
	eh.type=htons(0x0800); //IP Frames

	memcpy(&resetPacket,&eh,ETHER_LENGTH);


	
	/*******************  IP Header ******************/
	
	memset(&ih,0,sizeof(ih));
	
	//source address
	memcpy(&ih.saddr,devices[openDeviceIndex].ihaIp,4);
		
	//destination address
	memcpy(&ih.daddr,destIp,4);
	
	ih.proto=PROTO_TCP;         

	ih.flags_fo=htons(0x4000);			//For TCP Flag fixed
//	ih.flags_fo=htons(0x0000);			//For ICMP Flag fixed
	ih.identification=htons(0x0150);    //Any number
	
	
	ih.tlen=htons(IP_LENGTH+TCP_LENGTH);      //Depends upon data and tcp protocol
	ih.tos=0;
	ih.ttl=32;
	ih.ver_ihl=0x45;  //Version (v4) and  header length(5 nibbles)
	ih.crc=0;
	ih.crc=computeChecksum((u_short *)&ih,IP_LENGTH);

	memcpy(&resetPacket[ETHER_LENGTH],&ih,IP_LENGTH);

	
	
	
	/***************  TCP Header ****************************/

	  
	memset(&th,0,sizeof(th));
	th.sport=htons(srcPort);				// source port,
	th.dport=htons(destPort);				// dest port changes dynamically
	th.ackno=htonl(0);
	th.seqno=htonl(5555l);
	th.flag=TCP_RST;        // RST packet
	th.offset=0x50;         // data offset
	th.uptr=0;
	th.win=htons(512) ;
	th.checksum=0;
	
	//set up the pseudo header 
	memset(&psh,0,sizeof(psh));
	psh.saddr=ih.saddr;
	psh.daddr=ih.daddr;
	psh.proto=ih.proto;
	psh.zero=0;
	psh.tcp_len=htons(TCP_LENGTH);				//Length of Tcp header + Data in OCTATES
	memcpy(&psh.tcp,&th,TCP_LENGTH);
	
	th.checksum=computeChecksum((u_short*)&psh,TCP_LENGTH+PSEUDO_LENGTH);



	//copy new tcp header into pseudo header
	memcpy(&psh.tcp,&th,TCP_LENGTH);
		
	//Compute Checksum for TCP Header
	th.checksum=computeChecksum((u_short*)&psh,TCP_LENGTH+PSEUDO_LENGTH);

	//Copy TCP Header into the frame at the end of IP Packet
	memcpy(&resetPacket[ETHER_LENGTH+IP_LENGTH],&th,TCP_LENGTH);

	
	return error;
}



/*********************************************************************************\
*
\*********************************************************************************/
void Knocker::sendSynPacket		()
{
	//Send the packet
	pcap_sendpacket(hdev,synPacket,ETHER_LENGTH+IP_LENGTH+TCP_LENGTH+4);

}

/*********************************************************************************\
*
\*********************************************************************************/
void Knocker::sendResetPacket		()
{
	//Send the packet
	pcap_sendpacket(hdev,resetPacket,ETHER_LENGTH+IP_LENGTH+TCP_LENGTH);

}


/*********************************************************************************\
*
\*********************************************************************************/
bool Knocker::playSequence		()
{
	bool error;

	error=false;


	if (knockThreadId==NULL)
	{

		listenThreadId = CreateThread( 
			NULL,              // default security attributes
			0,                 // use default stack size  
			packetCatchThreadFunction,    // thread function 
			(LPVOID)this,      // argument to thread function 
			0,                 // use default creation flags 
			NULL);	           // returns the thread identifier 

		if (listenThreadId == NULL) 
		{
			printError(_T("Error starting listen thread"));
			error=true;
		}

		Sleep(1000);

		knockThreadId = CreateThread( 
			NULL,              // default security attributes
			0,                 // use default stack size  
			knockThreadFunction,    // thread function 
			(LPVOID)this,      // argument to thread function 
			0,                 // use default creation flags 
			NULL);	           // returns the thread identifier 

		// Check the return value for success. 

		if (knockThreadId == NULL) 
		{
			printError(_T("Error starting knock thread"));
			error=true;
		}
	}
	else
	{
		printError("Sequence busy!");
		error=true;
	}

	return error;
}


/*********************************************************************************\
*
\*********************************************************************************/
bool Knocker::mergeAdapterInfo()
{
	bool	error;
	int		i;
	int		ihaIndex;

	error=false;

	i=0;
	while ((i<deviceCount) && !error)
	{
		findTheAdapter(i, &ihaIndex);
		if (theAdapter!=NULL)
		{
			devices[i].ihaDeviceIndex	=ihaIndex;
			devices[i].ihaName			=theAdapter->AdapterName;
			devices[i].ihaDescription	=theAdapter->Description;
			memcpy(devices[i].ihaMac,theAdapter->Address,6);
			stringToByteArray(theAdapter->IpAddressList.IpAddress.String, devices[i].ihaIp            , 4, '.');
			stringToByteArray(theAdapter->IpAddressList.IpMask.String   , devices[i].ihaNetMask       , 4, '.');
			stringToByteArray(theAdapter->GatewayList.IpAddress.String  , devices[i].ihaGatewayIp     , 4, '.');
			stringToByteArray(theAdapter->GatewayList.IpMask.String     , devices[i].ihaGatewayNetMask, 4, '.');
			if (theAdapter->DhcpEnabled)
			{
				devices[i].ihaDhcpEnabled   =true;
			}
			else
			{
				devices[i].ihaDhcpEnabled   =false;
			}
			stringToByteArray(theAdapter->DhcpServer.IpAddress.String  , devices[i].ihaDhcpIp       , 4, '.');
			stringToByteArray(theAdapter->DhcpServer.IpMask.String     , devices[i].ihaDhcpNetMask  , 4, '.');
		}
		i++;
	}


	return error;
}


/*********************************************************************************\
*
\*********************************************************************************/
bool Knocker::getIphlpAdapterInfo()
{
	bool	error;
	DWORD	buflen;
	DWORD	status;

	error=false;


	buflen=sizeof(ihaAdapterInfo);
	// Get the adapter information. This is IP helper functionality (iphlpapi.h)
	status=GetAdaptersInfo(ihaAdapterInfo, &buflen);



	// If successful, process data
	if(status==ERROR_SUCCESS)
	{
		error=mergeAdapterInfo();
	}
	else
	{
		switch (status)
		{
		case ERROR_BUFFER_OVERFLOW:
			printError(_T("IPHLPAPI: buffer overflow: cannot get adapter info"));
			break;
		case ERROR_INVALID_PARAMETER:
			printError(_T("IPHLPAPI: parameter error: cannot get adapter info"));
			break;
		case ERROR_NO_DATA:
			printError(_T("IPHLPAPI: no data available: cannot get adapter info"));
			break;
		case ERROR_NOT_SUPPORTED:
			printError(_T("IPHLPAPI: not supported: cannot get adapter info"));
			break;
		}

		error=true;
	}

	return error;
}


/*********************************************************************************\
*
\*********************************************************************************/
int Knocker::getDeviceList()
{
	int			retvalue;
	bool		error;
	int			i;
	char		err[PCAP_ERRBUF_SIZE];
	pcap_if_t	*d;


	error=false;


    deviceCount=0;

	retvalue=pcap_findalldevs(&devlist, err);
	

	if(retvalue==-1 || devlist==NULL)
	{
		printError(_T("WINPCAP: cannot get device list"));
		error=true;
	}


	if (!error)
	{
		
		// Enumerate available devices...
		// Store the name and decription...
		for(d=devlist,i=0; d && i<10 ;d=d->next,i++)
		{
			devices[i].wpcName       =d->name;
			devices[i].wpcDescription=d->description;
		}
	
		deviceCount=i;


	}
	else
	{
		deviceCount=0;
	}


	// Add the info to the device list as obtained by IP Helper 
	error=getIphlpAdapterInfo();



	return deviceCount;

}

/*********************************************************************************\
*
\*********************************************************************************/
char** Knocker::getDeviceDescriptions()
{
	int i;

	i=0;
	while (i<deviceCount)
	{
		descriptions[i]=devices[i].wpcDescription;
		i++;
	}
	while (i<MAX_ADAPTER)
	{
		descriptions[i]='\0';;
		i++;
	}
	return descriptions;
}




//
//  Send ARP Request packet to determine the MAC address of
//  the destination machine.
//  ARP Reply message contains the MAC address of destination
//  machine...
//  This trick also prevents the target host from sending arp packet
//  to source host when source host sends the first SYN packet.
//  From the ARP request packet , target host will come to know
//  about the MAC address of the source host...
//

/*********************************************************************************\
*
\*********************************************************************************/
bool Knocker::getDestMacAddress()
{
	bool		error;
	bool		exit;
	eth_header	eh;
	eth_header	*peh;
	arp_header	arph;
	arp_header	*parph;

	int			retvalue;
	u_char		frame[200];
	char		str[100],filter[200];

	error=false;



	// First check if the destination IP is in the local network segment
	// If not, the gateway should be ARPed

	if (this->isInNetworkSegment())
	{
		targetMachine=destIp;
	}
	else
	{
		if (!error)
		{
			targetMachine=devices[openDeviceIndex].ihaGatewayIp;
		}
	}




	if (!error)
	{
		// Apply filter to receive only ARP reply packet...
		sprintf(str,"%d.%d.%d.%d", targetMachine[0], targetMachine[1], targetMachine[2], targetMachine[3]);
		sprintf(filter,"arp src net %s",str);
		applyFilter(filter);
		

		// Prepare ARP request packet

		/*******************  Ethernet Header ******************/

		memset(&eh, 0, sizeof(eh));
		
		// Source Mac address
		memcpy(eh.shost, devices[openDeviceIndex].ihaMac, 6);

		// Destination MAC address ; make it all 0xff
		memset(eh.dhost,0xff,6);
		
		eh.type=htons(0x0806); //IP Frames

		memcpy(&frame,&eh,ETHER_LENGTH);


		/*******************  ARP Header ******************/
		
		arph.eth_type=htons(0x0800);
		
		memcpy(arph.smac,eh.shost,6);
		memset(arph.dmac,0,6);

		memcpy(&arph.daddr, targetMachine, 4);
		memcpy(&arph.saddr, devices[openDeviceIndex].ihaIp, 4);
		
		arph.hrd=htons(0x0001);
		
		arph.iplen=4;
		arph.maclen=6;

		arph.opcode=0x0100;  //Request ARP packet

		memcpy(&frame[ETHER_LENGTH],&arph,ARP_LENGTH);

			
		//Send the ARP request packet
		pcap_sendpacket(hdev,frame,ETHER_LENGTH+ARP_LENGTH);

		
		//Now wait ARP reply packet
		int count=0;

		exit=false;
		while(!exit)
		{
			//Check if stop scan button is pressed
	/*
			if(stopscanning==TRUE)
			{
				stopscanning=FALSE;
				return FALSE;
			}
	*/

			retvalue=pcap_next_ex(hdev,&header,&pkt_data);
		
			if(retvalue<0)
			{
				printError(_T("Error in capturing the packet"));
				error=true;
				exit=true;
			}
			
			if(!error)
			{	 
				if (header->caplen==0)
				{
					count++;
					
					if(count>5)
					{
						error=true;
						exit=true;
					}
				
				}
				else
				{
					//Now this packet is the required packet..
					//So take a break and check it out....
				
					//check for ARP packet IP address checking is better


					//Get the Ethernet Header...
					peh=(eth_header*)pkt_data;

					if(ntohs(peh->type)==0x0806)	
					{
						// Get the ARP Header
						parph=(arp_header*)(pkt_data+ETHER_LENGTH);
					
						//check if it is Reply
						if(parph->opcode==0x0200)
						{
							memcpy(destMac,parph->smac,6);
							exit=true;
						}
					
					}
				}
			}

		}

		pcap_freecode(&fcode);		// reset the filter	
	}

	return error;
}

/*********************************************************************************\
*
\*********************************************************************************/
u_short Knocker::computeChecksum(u_short *data,int size)
{
	register int nleft=size;
	register u_short *w=data;
	register int sum=0;
	u_short answer =0;



	while(nleft>1)
	{
	sum+=*w++;
	nleft-=2;
	}


	if(nleft==1)
	{

		*(u_char*) (&answer) = *(u_char*)w;
		sum+=answer;
	}


	sum= (sum>>16 )+ (sum & 0xffff);
	sum+=(sum>>16);
	answer=~sum;
	return (answer);

}

/*********************************************************************************\
*
\*********************************************************************************/
LRESULT Knocker::applyFilter(char *filter)
{
	u_int netmask;
	int retvalue;

	//log.WriteString(strcat(mesg,filter));
	if(devlist->addresses!=NULL)
	{
		netmask=((struct sockaddr_in *)(devlist->addresses->netmask))->sin_addr.S_un.S_addr;
	}
	else
	{
		netmask=0xffffffff;
	}

	//compile the filter
	retvalue=pcap_compile(hdev,&fcode,filter,1,netmask);
	
	if(retvalue<0)
	{
		printError(_T("Unable to compile the filter"));
		return FALSE;
	}

	//Set the filter
	retvalue=pcap_setfilter(hdev,&fcode);
	
	if(retvalue<0)
	{
	printError(_T("Unable to set the filter"));
	return FALSE;
	}
	
	
	return TRUE;

}

/*********************************************************************************\
*
\*********************************************************************************/
bool Knocker::findTheAdapter(int wpcIndex, int* ihaIndex)
{
	int		i;
	bool	error;
	bool	found;
	char*	start;


	error=false;

	// Now find the adapter that is used (indicated by index)
	i=0;
	found=false;
	while ((i<MAX_ADAPTER) && !found)
	{
		// Compare descriptions. They appear to be identical in both lists
		start=strchr(devices[wpcIndex].wpcName, ihaAdapterInfo[i].AdapterName[0]);
		if (start!=NULL)
		{
			if (strcmp(start, ihaAdapterInfo[i].AdapterName)==0)
			{
				*ihaIndex=i;
				found=true;
			}
			else
			{
				i++;
			}
		}
		else
		{
			i++;
		}
	}
	if (found)
	{
		theAdapter=&ihaAdapterInfo[i];
	}
	else
	{
		theAdapter=NULL;
		error=true;
	}

	return error;
}



/*********************************************************************************\
*
\*********************************************************************************/
bool Knocker::isInNetworkSegment()
{
	bool isInSegment;
	int i;

	isInSegment=true;

	i=0;
	while (i<4)
	{
		if ((destIp[i] ^ devices[openDeviceIndex].ihaIp[i]) & devices[openDeviceIndex].ihaNetMask[i])
		{
			isInSegment=false;
		}
		i++;
	}

	return isInSegment;
}


/*********************************************************************************\
*
\*********************************************************************************/
bool Knocker::stringToByteArray(char*  string, BYTE* array, int arrayLength, char delimiter)
{
	bool	error;
	char*	stringPtr;
	char*	nextPeriod;
	int		i;

	error=false;

	stringPtr=string;
	i=0;
	while ((i<arrayLength-1) && (!error))
	{
		nextPeriod=strchr(stringPtr, delimiter);
		if (nextPeriod!=NULL)
		{
			*nextPeriod='\0';
			array[i]=(BYTE)atoi(stringPtr);
			stringPtr=nextPeriod+1;
		}
		else
		{
			error=true;
		}
		i++;
	}

	if (!error)
	{
		array[i]=(BYTE)atoi(stringPtr);
	}


	return error;
}

/*********************************************************************************\
*
\*********************************************************************************/

char* Knocker::tcharToChar(TCHAR* string, int length)
{
	int i;

	i=0;
	while (i<length)
	{
		conversionString[i]=(char)string[i];
		i++;
	}
	return conversionString;
}






/*********************************************************************************\
*
\*********************************************************************************/
TCHAR* Knocker::getInterfaceInfo()
{
	int i;
	
	// Clear string
	infoInChar[0]='\0';

	sprintf(dummy, "ADAPTER INFO\n");
	strncat(infoInChar, dummy, MAX_INFO);

	i=0;
	while (i<deviceCount)
	{
		sprintf(dummy, "Adapter %3d    : %s\n", i, devices[i].wpcName);
		strncat(infoInChar, dummy, MAX_INFO);
		sprintf(dummy, "IHA Name       : %s\n", devices[i].ihaName);
		strncat(infoInChar, dummy, MAX_INFO);
		sprintf(dummy, "Description    : %s\n", devices[i].wpcDescription);
		strncat(infoInChar, dummy, MAX_INFO);
		sprintf(dummy, "IHA Description: %s\n", devices[i].ihaDescription);
		strncat(infoInChar, dummy, MAX_INFO);

		sprintf(dummy, "MAC address    : %02X:%02X:%02X:%02X:%02X:%02X\n",	
															devices[i].ihaMac[0],
															devices[i].ihaMac[1],
															devices[i].ihaMac[2],
															devices[i].ihaMac[3],
															devices[i].ihaMac[4],
															devices[i].ihaMac[5]);
		strncat(infoInChar, dummy, MAX_INFO);

		sprintf(dummy, "IP  address    : %d.%d.%d.%d/%d.%d.%d.%d\n",	
												devices[i].ihaIp[0],
												devices[i].ihaIp[1],
												devices[i].ihaIp[2],
												devices[i].ihaIp[3],
												devices[i].ihaNetMask[0],
												devices[i].ihaNetMask[1],
												devices[i].ihaNetMask[2],
												devices[i].ihaNetMask[3]);
		strncat(infoInChar, dummy, MAX_INFO);
		
		sprintf(dummy, "Gateway        : %d.%d.%d.%d/%d.%d.%d.%d\n",	
												devices[i].ihaGatewayIp[0],
												devices[i].ihaGatewayIp[1],
												devices[i].ihaGatewayIp[2],
												devices[i].ihaGatewayIp[3],
												devices[i].ihaGatewayNetMask[0],
												devices[i].ihaGatewayNetMask[1],
												devices[i].ihaGatewayNetMask[2],
												devices[i].ihaGatewayNetMask[3]);
		strncat(infoInChar, dummy, MAX_INFO);

		if (devices[i].ihaDhcpEnabled)
		{
			sprintf(dummy, "DHCP Server    : %d.%d.%d.%d/%d.%d.%d.%d\n",	
													devices[i].ihaDhcpIp[0],
													devices[i].ihaDhcpIp[1],
													devices[i].ihaDhcpIp[2],
													devices[i].ihaDhcpIp[3],
													devices[i].ihaDhcpNetMask[0],
													devices[i].ihaDhcpNetMask[1],
													devices[i].ihaDhcpNetMask[2],
													devices[i].ihaDhcpNetMask[3]);
			strncat(infoInChar, dummy, MAX_INFO);
		}
		
		strncat(infoInChar, "\n", MAX_INFO);		
		i++;
	}


/*
	sprintf(dummy, "IPHLPAPI INFO\n");
	strncat(infoInChar, dummy, MAX_INFO);


	i=0;
	while (i<MAX_ADAPTER)
	{
		if (strcmp(ihaAdapterInfo[i].AdapterName, "")!=0)
		{
			sprintf(dummy, "Adapter %3d: %s\n", i, ihaAdapterInfo[i].AdapterName);
			strncat(infoInChar, dummy, MAX_INFO);
			sprintf(dummy, "Description: %s\n", ihaAdapterInfo[i].Description);
			strncat(infoInChar, dummy, MAX_INFO);
			sprintf(dummy, "MAC address: %02X:%02X:%02X:%02X:%02X:%02X\n",	
																ihaAdapterInfo[i].Address[0],
																ihaAdapterInfo[i].Address[1],
																ihaAdapterInfo[i].Address[2],
																ihaAdapterInfo[i].Address[3],
																ihaAdapterInfo[i].Address[4],
																ihaAdapterInfo[i].Address[5]);
			strncat(infoInChar, dummy, MAX_INFO);
			
			sprintf(dummy, "IP  address: %s/%s\n",	ihaAdapterInfo[i].IpAddressList.IpAddress.String,
													ihaAdapterInfo[i].IpAddressList.IpMask.String);
			strncat(infoInChar, dummy, MAX_INFO);
			
			sprintf(dummy, "Gateway     : %s/%s\n",	ihaAdapterInfo[i].GatewayList.IpAddress.String,
													ihaAdapterInfo[i].GatewayList.IpMask.String);
			strncat(infoInChar, dummy, MAX_INFO);

			if (ihaAdapterInfo[i].DhcpEnabled)
			{
				sprintf(dummy, "DHCP server : %s/%s\n",	ihaAdapterInfo[i].DhcpServer.IpAddress.String,
														ihaAdapterInfo[i].DhcpServer.IpMask.String);
				strncat(infoInChar, dummy, MAX_INFO);
			}

			strncat(infoInChar, "\n", MAX_INFO);
		}
		i++;
	}


	sprintf(dummy, "WINPCAP INFO\n");
	strncat(infoInChar, dummy, MAX_INFO);


	i=0;
	while (i<deviceCount)
	{
		sprintf(dummy, "Adapter %3d: %s\n", i, devices[i].wpcName);
		strncat(infoInChar, dummy, MAX_INFO);
		sprintf(dummy, "Description: %s\n", devices[i].wpcDescription);
		strncat(infoInChar, dummy, MAX_INFO);
		i++;
	}

*/

	// Now convert char string to TCHAR string
	i=0;
	while (i<int(strlen(infoInChar)+1))
	{
		info[i]=(TCHAR)infoInChar[i];
		i++;
	}

	return info;
}

