// Knocker.h: interface for the Knocker class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_KNOCKER_H__A9C5FF14_E362_4EA8_9934_5C8BD030D16C__INCLUDED_)
#define AFX_KNOCKER_H__A9C5FF14_E362_4EA8_9934_5C8BD030D16C__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "pcap.h"




#define  s6_addr 11111     //This trick is to eliminate redefinition
						   //of in6_addr in iphlpapi.h file since it is 
						   //already defined in ws2tcpip.h
#include <iphlpapi.h>


#pragma comment(lib,"iphlpapi.lib")





//  define the tcp flags....
#define   TCP_FIN		0x01
#define   TCP_SYN		0x02
#define   TCP_RST		0x04
#define   TCP_PSH		0x08
#define   TCP_ACK		0x10
#define   TCP_URG		0x20
#define   TCP_ACE		0x40
#define   TCP_CWR		0x80


//define header lengths

#define ETHER_LENGTH	14
#define IP_LENGTH		20
#define TCP_LENGTH		20
#define UDP_LENGTH		8
#define ICMP_LENGTH		8
#define ARP_LENGTH		28
#define PSEUDO_LENGTH	12
#define DATA_LENGTH		32


//Protocol
#define PROTO_ICMP		1
#define PROTO_IGMP		2
#define PROTO_TCP		6
#define PROTO_UDP		17

#define MAX_SEQUENCE		4
#define SEQUENCE_MINVALUE	0
#define SEQUENCE_MAXVALUE	9


#define MAX_ADAPTER			10

#define MAX_INFO			1000

#define TIMEWINDOW_MIN		1000
#define TIMEWINDOW_MAX		9000

#define PORT_MIN			1
#define PORT_MAX			65535


struct eth_header   // 14 bytes
{
	u_char dhost[6]; //destination mac address
	u_char shost[6]; //source mac address
	u_short type;                 //IP ,ARP , RARP
};
	
	
// 4 byte ip address
struct ip_address
{
	u_char byte1;
	u_char byte2;
	u_char byte3;
	u_char byte4;
};


/* IPv4 header */
   struct ip_header
{
    u_char  ver_ihl;        // Version (4 bits) + Internet header length (4 bits)
    u_char  tos;            // Type of service 
    u_short tlen;           // Total length 
    u_short identification; // Identification
    u_short flags_fo;       // Flags (3 bits) + Fragment offset (13 bits)
    u_char  ttl;            // Time to live
    u_char  proto;          // Protocol
    u_short crc;            // Header checksum
    ip_address  saddr;      // Source address
    ip_address  daddr;      // Destination address
   
	// u_int   op_pad;         // Option + Padding
};

	
	
struct arp_header   //28 bytes
{
	u_short hrd;       //hardware address space=0x0001
	u_short eth_type;  //Ethernet type ....=0x0800
	u_char maclen;     //Length of mac address=6
	u_char iplen;      //Length of ip addres=4
	u_short opcode;    //Request =1 Reply=2 (highbyte)
	u_char smac[6];    //source mac address
	ip_address saddr;  //Source ip address
	u_char dmac[6];    //Destination mac address
	ip_address daddr;  //Destination ip address
};
	

struct udp_header   //8 bytes
{
    u_short sport;          // Source port
    u_short dport;          // Destination port
    u_short len;            // Datagram length
    u_short crc;            // Checksum
};

struct tcp_header  //20 bytes
{
	u_short sport;      //Source port
	u_short dport;      //Destination port
	u_long seqno;       //Sequence no
	u_long ackno;       //Ack no
	u_char offset;      //Higher level 4 bit indicates data offset
	u_char flag;        //Message flag
					//FIN - 0x01
					//SYN - 0x02
					//RST - 0x04 
					//PUSH- 0x08
					//ACK- 0x10
					//URG- 0x20
					//ACE- 0x40
					//CWR- 0x80

	u_short win;
	u_short checksum;
	u_short uptr;
};

struct icmp_header
{
	u_char type;
	u_char code;
	u_short checksum;
	u_short id;
	u_short seqno;
};
	
//For checksum calculation purpose
struct pseudo_header  //12 bytes 
{
    ip_address  saddr;      // Source address
    ip_address  daddr;      // Destination address
	u_char zero;
	u_char  proto;          // Protocol
	u_short tcp_len;
	tcp_header tcp;
};


typedef struct
{
	char*	wpcName;
	char*	wpcDescription;
	char*   ihaName;
	char*	ihaDescription;
	BYTE	ihaMac[8];
	BYTE	ihaIp[4];
	BYTE	ihaNetMask[4];
	BYTE	ihaGatewayIp[4];
	BYTE	ihaGatewayNetMask[4];
	bool	ihaDhcpEnabled;
	BYTE	ihaDhcpIp[4];
	BYTE	ihaDhcpNetMask[4];
	int		ihaDeviceIndex;
} deviceInfo_t;


typedef enum
{
	SYNPACKET,
	RSTPACKET
} packet_t;

class Knocker  
{
private:
	TCHAR				info[MAX_INFO];
	char				infoInChar[MAX_INFO];
	char				dummy[MAX_INFO];

	IP_ADAPTER_INFO		ihaAdapterInfo[MAX_ADAPTER];			// adapter info as generated by IP Helper (iphlpapi.h)
																// this one includes the MAC address
	IP_ADAPTER_INFO*	theAdapter;	


	pcap_if_t			*devlist;						// adapter info as generated by WinPcap
	
	int					deviceCount;					// number of adapters found (WinPcap)
	deviceInfo_t		devices[MAX_ADAPTER];			// all necessary information

	char*				descriptions[MAX_ADAPTER];

	int					timeWindow;

	BYTE				destMac[8];
	BYTE				destIp[4];

	BYTE				knockSequence[MAX_SEQUENCE];

	int					destPort;

	
	BYTE*		targetMachine;

	pcap_t				*hdev;
	struct pcap_pkthdr	*header;
	struct bpf_program	fcode;
	const u_char		*pkt_data;

	bool				isOpen;
	int					openDeviceIndex;

	u_char				synPacket[200];
	u_char				resetPacket[200];

	TCHAR				status[100];

	HANDLE				knockThreadId;
	HANDLE				listenThreadId;

	char				conversionString[100];
	TCHAR				macString[25];
	
private:			
	/**
	 *  This method requests the adapter information from IP Helper
	 *  @return False if successful, True if an error occurred.
	 */
	bool		getIphlpAdapterInfo		();
	
	bool		findTheAdapter			(int wpcIndex, int* ihaIndex);
	
	bool		isInNetworkSegment		();

//	bool		stringToByteArray		(String string, BYTE* array, int arrayLength, char delimiter);
	bool		stringToByteArray		(TCHAR* string, BYTE* array, int arrayLength, char delimiter);

	char*		tcharToChar				(TCHAR* string, int length);

	bool		mergeAdapterInfo		();
	bool		mergeAdapterAddresses	();
public:
				Knocker					();
	virtual		~Knocker				();

	bool		setDestinationIp		(TCHAR* ip);
	bool		setDestinationPort		(TCHAR* port);
	bool		setKnockSequence		(TCHAR* sequence);
	bool		setTimeWindow			(TCHAR* window);
	bool		setMacAddress			(int index, TCHAR* macAddress);
	TCHAR*		getMacAddress			(int index);

	char**		getDeviceDescriptions	();

	bool		preparePacket			();
	bool		createSynPacket			();
	bool		createResetPacket		(int srcPort);
	void		sendSynPacket			();
	void		sendResetPacket			();
	bool		playSequence			();


	int			getDeviceList			();
	LRESULT		applyFilter				(char *filter);
	u_short		computeChecksum			(u_short *data,int size);
	bool		getDestMacAddress		();

	bool		openDevice				(int index);
	bool		closeDevice				();

	TCHAR*		getInterfaceInfo		();

	friend DWORD WINAPI knockThreadFunction(LPVOID knockerInstance);
	friend DWORD WINAPI packetCatchThreadFunction(LPVOID knockerInstance);

};



#endif // !defined(AFX_KNOCKER_H__A9C5FF14_E362_4EA8_9934_5C8BD030D16C__INCLUDED_)
