// OpenSesamy.cpp : Defines the entry point for the application.
//

#include "stdafx.h"
#include "resource.h"

#include "knocker.h"


// Global Variables:
Knocker*	knocker=NULL;
HWND		hDialogbox=NULL;
HINSTANCE	theInstance=NULL;

TCHAR		destinationPc[200];
TCHAR		destinationPort[100];
TCHAR		sequence[100];
TCHAR		timeWindow[100];
int			deviceIndex;
TCHAR		macAddress[100];

// Foward declarations of functions included in this code module:
LRESULT CALLBACK dialogHandler(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);



/*********************************************************************************\
*
\*********************************************************************************/
int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     int       nCmdShow)
{

	theInstance=hInstance;

	DialogBox(NULL, (LPCTSTR)IDD_OPENSESAMY, NULL, (DLGPROC)dialogHandler);

	return 0;
}




/*********************************************************************************\
*
\*********************************************************************************/
void updateStatus(TCHAR* string)
{
	if (hDialogbox!=NULL)
	{
		SetDlgItemText(hDialogbox, IDC_STATUS, string);
	}
}

/*********************************************************************************\
*
\*********************************************************************************/
void printError(TCHAR* string)
{
	MessageBox(NULL, string, _T("Error"), 0);
}


/*********************************************************************************\
*
\*********************************************************************************/

void loadSettings()
{
	FILE *file;

	file=_tfopen("knocksettings.txt", "rt");

	if (file != NULL)
	{
		_ftscanf(file, "%s\n", destinationPc);
		_ftscanf(file, "%s\n", destinationPort);
		_ftscanf(file, "%s\n", sequence);
		_ftscanf(file, "%s\n", timeWindow);
		_ftscanf(file, "%d\n", &deviceIndex);
		_ftscanf(file, "%s\n", macAddress);
		fclose(file);
	}
}

/*********************************************************************************\
*
\*********************************************************************************/
LRESULT CALLBACK infoDialogHandler(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
		case WM_INITDIALOG:
			SetDlgItemText(hDlg, IDC_INTERFACEINFO, knocker->getInterfaceInfo());
			return TRUE; 

		case WM_COMMAND:
			if (LOWORD(wParam) == IDOK)
			{
				EndDialog(hDlg, LOWORD(wParam));
				return TRUE;
			}
			break;
	}
    return FALSE;
}

/*********************************************************************************\
*
\*********************************************************************************/
LRESULT CALLBACK aboutDialogHandler(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
		case WM_INITDIALOG:
			return TRUE; 

		case WM_COMMAND:
			if (LOWORD(wParam) == IDOK)
			{
				EndDialog(hDlg, LOWORD(wParam));
				return TRUE;
			}
			break;
	}
    return FALSE;
}


/*********************************************************************************\
*
\*********************************************************************************/
LRESULT CALLBACK dialogHandler(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	char**			devices;
	int				noOfDevices;
	int				i;
	TCHAR			buffer[150];
	int				index;
	bool			error;
	static bool		opened=false;
	TCHAR*			text;

	switch (message)
	{
		case WM_INITDIALOG:
			hDialogbox=hDlg;
			knocker=new Knocker();


			// Add the devices to the device list
			noOfDevices=knocker->getDeviceList();
			if (noOfDevices>0)
			{
				devices=knocker->getDeviceDescriptions();

				i=0;
				while (i<noOfDevices)
				{
					_stprintf(buffer, _T("%s"), devices[i]);
					SendDlgItemMessage(hDlg, IDC_DEVICES, CB_INSERTSTRING, -1, (LPARAM)buffer);
					i++;
				}

				updateStatus(_T("Fill in fields, choose device and Knock"));
			}
			else
			{
				// don't even think about opening the dialog...
//				delete knocker;
//				hDialogbox=NULL;
//				EndDialog(hDlg, LOWORD(wParam));

			}

			// If present, load settings
			loadSettings();
			SetDlgItemText(hDlg, IDC_SERVER, destinationPc); 
			SetDlgItemText(hDlg, IDC_PORT, destinationPort); 
			SetDlgItemText(hDlg, IDC_SEQUENCE, sequence); 
			SetDlgItemText(hDlg, IDC_TIMEWINDOW, timeWindow); 
			SendDlgItemMessage(hDlg, IDC_DEVICES, CB_SETCURSEL, (WPARAM)deviceIndex, 0);
			SetDlgItemText(hDlg, IDC_MAC, macAddress); 


			return TRUE; 

		case WM_COMMAND:
			if (LOWORD(wParam) == ID_KNOCK)
			{
				//
				// Pass the parameters from the dialog box to the Knocker instance
				//
				GetDlgItemText(hDlg, IDC_SERVER, buffer, 150);
				error=knocker->setDestinationIp(buffer);
				if (error)
				{
					updateStatus(_T("Error in address!"));
				}
				
				if (!error)
				{
					GetDlgItemText(hDlg, IDC_PORT, buffer, 150);
					error=knocker->setDestinationPort(buffer);
					if (error)
					{
						updateStatus(_T("Error in port!"));
					}
				}

				if (!error)
				{
					GetDlgItemText(hDlg, IDC_TIMEWINDOW, buffer, 150);
					error=knocker->setTimeWindow(buffer);
					if (error)
					{
						updateStatus(_T("Error in port!"));
					}
				}


				if (!error)
				{
					GetDlgItemText(hDlg, IDC_SEQUENCE, buffer, 150);
					error=knocker->setKnockSequence(buffer);
					if (error)
					{
						updateStatus(_T("Error in sequence!"));
					}
				}

				if (!error)
				{
					index=SendDlgItemMessage(hDlg, IDC_DEVICES, CB_GETCURSEL, 0, 0);
					if (index<0)
					{
						updateStatus(_T("Please choose network adapter"));
						error=true;
					}
				}
				
				if (!error)
				{
					GetDlgItemText(hDlg, IDC_MAC, buffer, 150);
					error=knocker->setMacAddress(index, buffer);
					if (error)
					{
						updateStatus(_T("Error in sequence!"));
					}
				}
				//
				// Now, if all went well, do the knocking...
				//
				if (!error && !opened)
				{
					error=knocker->openDevice(index);
					opened=true;
				}

				if (!error)
				{
					error=knocker->preparePacket();
				}
				
				if (!error)
				{
					error=knocker->playSequence();
				}

			}
			else if ((LOWORD(wParam) == IDC_DEVICES) && (HIWORD(wParam) == CBN_SELENDOK))
			{
				index=SendDlgItemMessage(hDlg, IDC_DEVICES, CB_GETCURSEL, 0, 0);
				if (index>=0)
				{
					text=knocker->getMacAddress(index);
					SetDlgItemText(hDlg, IDC_MAC, text);
				}
			}
			else if (LOWORD(wParam) == ID_INFO)
			{
				DialogBox(NULL, (LPCTSTR)IDD_INTERFACEINFO, NULL, (DLGPROC)infoDialogHandler);
			}
			else if (LOWORD(wParam) == ID_ABOUT)
			{
				DialogBox(theInstance, (LPCTSTR)IDD_ABOUTBOX, NULL, (DLGPROC)aboutDialogHandler);
			}
			else if (LOWORD(wParam) == ID_CANCEL)
			{
				if (opened)
				{
					knocker->closeDevice();
					opened=false;
				}
				delete knocker;
				hDialogbox=NULL;
				EndDialog(hDlg, LOWORD(wParam));
				return TRUE;
			}
			break;
	}
    return FALSE;
}
